<?php
/**
* Tag Groups
*
* @package     Tag Groups
* @author      Christoph Amthor
* @copyright   2019 Christoph Amthor (@ Chatty Mango, chattymango.com)
* @license     GPL-3.0+
*
*/

if ( ! class_exists( 'TagGroups_Premium_Loader' ) ) {

  class TagGroups_Premium_Loader {


    /**
    * absolute path to the plugin main file
    *
    * @var string
    */
    var $plugin_path;


    function __construct( $plugin_path ) {

      $this->plugin_path = $plugin_path;

    }


    /**
    * Adds all required classes
    *
    * @param void
    * @return object $this
    */
    public function require_classes()
    {

      /*
      * Require all classes of this plugin
      */
      foreach ( glob( $this->plugin_path . '/include/entities/*.php') as $filename ) {

        require_once $filename;

      }

      foreach ( glob( $this->plugin_path . '/include/helpers/*.php') as $filename ) {

        require_once $filename;

      }

      foreach ( glob( $this->plugin_path . '/include/admin/*.php') as $filename ) {

        require_once $filename;

      }

      /**
      * Must be after helpers
      */
      foreach ( glob( $this->plugin_path . '/include/shortcodes/*.php') as $filename ) {

        require_once $filename;

      }

      return $this;

    }


    /**
    * Provide objects that we'll need frequently
    *
    * @param void
    * @return object $this
    */
    public function provide_globals() {

      global $tag_group_posts, $tag_group_premium_groups, $tag_group_premium_terms;

      $tag_group_posts = new TagGroups_Premium_Posts();

      $tag_group_premium_groups = new TagGroups_Premium_Groups();

      $tag_group_premium_terms = new TagGroups_Premium_Terms();

      return $this;

    }


    /**
    * Checks if the preconditions are met
    *
    * @param void
    * @return void
    */
    public function check_preconditions() {

      global $wp_version;

      // Check the minimum WP version
      if ( version_compare( $wp_version, TAG_GROUPS_MINIMUM_VERSION_WP , '<' ) ) {

        include_once( ABSPATH . 'wp-admin/includes/plugin.php' );

        error_log( '[Tag Groups Premium] Insufficient WordPress version for Tag Groups Premium plugin.' );

        update_option( 'tag_group_admin_notice', array(
          'type' => 'error',
          'content' => sprintf( __( 'The plugin %s requires WordPress %s to function properly. ', 'tag-groups-premium' ), '<b>Tag Groups Premium</b>', TAG_GROUPS_MINIMUM_VERSION_WP ) .
          __( 'Please upgrade WordPress and then try again.', 'tag-groups-premium' )
        ) );
        add_action('admin_notices', array('TagGroups_Base','admin_notice') );

        deactivate_plugins( TAG_GROUPS_PLUGIN_BASENAME );

        return false;

      }

      return $this;

    }


    /**
    * adds all hooks
    *
    * @param void
    * @return object $this
    */
    public function add_hooks()
    {

      global $tag_groups_premium_fs_sdk, $tag_group_posts;


      // backend
      add_action( 'admin_init', array( $this, 'add_admin_init_hooks') );


      if ( $tag_groups_premium_fs_sdk->is_plan_or_trial('premium') ) {

        /**
        * Check if the default tag box is disabled. If yes, remove the tags.
        */
        $tag_group_hide_tagsdiv = get_option( 'tag_group_hide_tagsdiv', 0 );

        if ( $tag_group_hide_tagsdiv ) {

          add_action( 'admin_init' , array( 'TagGroups_Premium_Meta_Box', 'remove_terms_from_post_data' ) );

        }

      }

      // adding premium content to the free plugin
      add_action( 'tag_groups_hook_shortcodes', array( 'TagGroups_Premium_Settings', 'premium_shortcodes' ) );


      // hooks for object caching
      $tag_group_object_cache = get_option( 'tag_group_object_cache', TagGroups_Cache::WP_OPTIONS );

      if ( 0 !== $tag_group_object_cache ) {

        add_filter( 'tag_groups_hook_cache_get', array( 'TagGroups_Premium_Cache', 'get' ), 10, 2 );

        add_action( 'tag_groups_hook_cache_set', array( 'TagGroups_Premium_Cache', 'set' ), 10, 2 );

      }


      /**
      * If a post meta was saved through quick edit, we must process the post meta.
      * Note: Moved out of admin hooks so that it fires also through API access
      * WP: "REST API requests are considered frontend requests"
      */
      add_action( 'save_post', array( $tag_group_posts, 'conditional_process_post' ), 10, 2 );

      /**
      * If a post meta was saved, DPF, DPFT and Post List will change their post inventory
      */
      add_action( 'purge_post_list_transients', array( 'TagGroups_Premium', 'purge_post_list_transients' ), 10, 2 );


      if ( is_admin() ) {

        $this->add_admin_hooks();

      } else {

        $this->add_frontend_hooks();

      }


      return $this;

    }


    /**
    * adds all hooks that need to be registered after init
    *
    * @param void
    * @return object $this
    */
    public function add_admin_init_hooks()
    {

      if ( ! is_admin() ) {

        return false;

      }

      global $tag_groups_premium_fs_sdk;

      if ( $tag_groups_premium_fs_sdk->is_plan_or_trial('premium') ) {

        /**
        * Hook into action to save information from the meta box
        */

        $tag_group_meta_box_taxonomies = get_option( 'tag_group_meta_box_taxonomy', array() );

        $tag_group_post_types = TagGroups_Taxonomy::post_types_from_taxonomies( $tag_group_meta_box_taxonomies );

        if ( is_array( $tag_group_post_types ) ) {

          foreach ( $tag_group_post_types as $post_type ) {

            /**
            * save_post_ fires AFTER a post has been saved
            */
            add_action( 'save_post_' . $post_type, array( 'TagGroups_Premium_Meta_Box', 'save_meta_box' ), 10, 2 );

          }

        }

      }

      // /**
      // * Add the Settings link in the plugin list
      // */
      // already done through free plugin
      // add_filter( "plugin_action_links_" . TAG_GROUPS_PLUGIN_BASENAME, array( 'TagGroups_Premium_Settings', 'add_plugin_settings_link' ) );

      return $this;

    }


    /**
    * Adds hooks for admin only
    *
    * @param void
    * @return void
    */
    public function add_admin_hooks()
    {

      if ( ! is_admin() ) {

        return false;

      }

      global $tag_groups_premium_fs_sdk, $tag_group_posts, $tag_group_premium_terms;

      $tag_group_meta_box_taxonomies = get_option( 'tag_group_meta_box_taxonomy', array() );

      if ( ! empty( $tag_group_meta_box_taxonomies ) ) {

        // Adding the metabox to the new/edit post screen
        add_action( 'load-post.php', array( 'TagGroups_Premium_Meta_Box', 'add_meta_box_hook' ) );

        add_action( 'load-post-new.php', array( 'TagGroups_Premium_Meta_Box', 'add_meta_box_hook' ) );

      }

      // adding premium content to the free plugin
      add_action( 'tag_groups_hook_settings_action', array( 'TagGroups_Premium_Settings', 'premium_settings_action' ) );

      add_filter( 'tag_groups_setting_topics', array( 'TagGroups_Premium_Settings', 'tag_groups_setting_topics' ) );

      add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_scripts_admin') );

      if ( $tag_groups_premium_fs_sdk->is_plan_or_trial('premium') ) {

        add_action( 'wp_ajax_tg_ajax_get_terms_by_group', array( 'TagGroups_Premium_Meta_Box', 'ajax_get_terms_by_group' ) );

        add_action( 'wp_ajax_tg_ajax_get_terms_for_post', array( 'TagGroups_Premium_Meta_Box', 'ajax_get_terms_for_post' ) );

        if ( get_option( 'tag_group_hide_tagsdiv', 0 ) ) {

          add_action( 'admin_menu', array( 'TagGroups_Premium_Meta_Box','remove_tag_meta_boxes' ) );

        }

      }

      add_filter( 'set-screen-option', array( 'TagGroups_Premium_Admin', 'set_screen_option' ), 10, 3);

      /**
      * Modify the tabs on admin settings pages
      */
      TagGroups_Premium_Settings::register_admin_structure_filter();

      /**
      * Modify the tabs on admin settings pages
      */
      TagGroups_Premium_Settings::register_tab_filters();

      /**
      * Invalidation and rebuilding of cache depending on actions
      */

      // ### Terms ###

      /**
      * After a term has been deleted, we must update the post meta
      */
      // add_action( 'delete_term', array( $tag_group_posts, 'update_post_meta_for_term' ), 10, 2 );
      // Will be run with groups_of_term_saved

      /**
      * After a term has been deleted, we might have to update the object cache
      */
      add_action( 'delete_term', array( 'TagGroups_Premium_Admin', 'encourage_cleaning_object_cache' ), 20 );

      /**
      * After a term has changed its groups, we must update the post count for this term.
      */
      add_action( 'groups_of_term_saved', array( $tag_group_premium_terms, 'fill_post_counts_transients' ), 12 );

      /**
      * After a term has changed its groups, we might have to update the object cache
      */
      add_action( 'groups_of_term_saved', array( 'TagGroups_Premium_Admin', 'encourage_cleaning_object_cache' ), 20 );

      /**
      * After a term has changed its groups, we must update the array of post tags.
      */
      add_action( 'groups_of_term_saved', array( $tag_group_posts, 'clear_tag_groups_post_terms' ), 11 );

      /**
      * After a term has changed its groups, we must update the post meta.
      */
      add_action( 'groups_of_term_saved', array( $tag_group_posts, 'update_post_meta_for_term' ), 10, 2 );


      // ### Groups ###

      /**
      * After a term group has been deleted, the tag_groups_post_counts array (per group) is invalid
      */
      add_action( 'term_group_deleted', array( $tag_group_premium_terms, 'renew_tag_groups_post_counts_cache' ) );

      /**
      * After a term group has been deleted, we might have to update the object cache
      */
      add_action( 'term_group_deleted', array('TagGroups_Premium_Admin', 'encourage_cleaning_object_cache' ), 20 );

      /**
      * After a term group has been deleted, we must update the array of post tags.
      */
      add_action( 'term_group_deleted', array( $tag_group_posts, 'clear_tag_groups_post_terms' ) );

      /**
      * After a term group has been deleted, we must update the post meta with the post tags.
      */
      add_action( 'term_group_deleted', array( $tag_group_posts, 'fix_all_incorrect_post_terms' ) );

      /**
      * After a term group has been saved (with a new name or position), we might have to update the object cache
      */
      add_action( 'term_group_saved', array('TagGroups_Premium_Admin', 'encourage_cleaning_object_cache' ), 20 );



      // ### Taxonomies ###

      /**
      * After the taxonomies have been changed, we must parse the post meta of affected post_type and run maintenance
      */
      add_action( 'taxonomies_saved', array( $tag_group_posts, 'fix_all_incorrect_post_terms' ), 11 );

      /**
      * After the taxonomies have been changed, the tag_groups_post_counts array (per group) is invalid
      */
      add_action( 'taxonomies_saved', array( $tag_group_premium_terms, 'renew_tag_groups_post_counts_cache' ), 12 );

      /**
      * After the taxonomies have been changed, the tag_groups_post_counts array (per group) is invalid
      */
      add_action( 'taxonomies_saved', array( 'TagGroups_Premium_Admin', 'encourage_cleaning_object_cache' ), 20 );



      // ### Posts ###

      /**
      * After the published status changed, we must update the post counts cache (a post might now be offline).
      */
      add_action( 'transition_post_status', array( $tag_group_premium_terms, 'renew_tag_groups_post_counts_cache' ), 10 );

      /**
      * After the published status changed, we might have to update the object cache.
      */
      // overkill? We don't want to bother the users when ever a post is published
      // add_action( 'transition_post_status', array( 'TagGroups_Premium_Admin', 'encourage_cleaning_object_cache' ), 20 );


      /**
      * Clearing the object cache via admin notice
      */
      add_action( 'wp_ajax_tg_ajax_clear_object_cache', array( 'TagGroups_Premium_Admin', 'clear_object_cache' ) );

      /**
      * Processing routines in chunks with process bar
      */
      add_action( 'wp_ajax_tg_ajax_process', array( 'TagGroups_Premium_Process', 'tg_ajax_process' ) );


    }


    /**
    * Adds hooks for the frontend only
    *
    * @param void
    * @return void
    */
    public function add_frontend_hooks()
    {

      global $tag_groups_premium_fs_sdk, $tag_group_posts, $tag_group_premium_terms;

      add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );

      add_action( 'pre_get_posts', array( $tag_group_premium_terms, 'term_archive_filter') );


      if ( $tag_groups_premium_fs_sdk->is_plan_or_trial('premium') ) {

        /**
        * Add a table under the content to display tag groups and tags
        */
        $tag_group_display_groups_under_posts_priority = get_option( 'tag_group_display_groups_under_posts_priority', 10 );

        add_filter( 'the_content', array( $tag_group_posts, 'add_groups_and_tags_after_content' ), $tag_group_display_groups_under_posts_priority );


        /**
        * WooCommerce: replace template so that we can display tag groups as attributes
        */
        add_action('woocommerce_product_additional_information', array( $tag_group_posts, 'wc_display_product_attributes' ), 1 );


        /**
        * WooCommerce: show the "Additional Information" tab, even if there are no native WC Attributes
        */
        add_filter( 'wc_product_enable_dimensions_display', array( $tag_group_posts, 'wc_display_additional_information' ) );

        /**
        * Remove the post tags on the front end
        */
        if ( get_option( 'tag_group_remove_the_post_terms', false ) ) {

          $taxonomies = get_option( 'tag_group_taxonomy', array( 'post_tag' ) );

          foreach ( $taxonomies as $taxonomy ){

            if ( 'product_tag' === $taxonomy) {

              // It is more complicated for WooCommerce - we use our own menu.
              continue;

            }

            add_filter( 'term_links-' . $taxonomy, array( $tag_group_posts, 'remove_the_post_terms' ) );

          }

        }


        /**
        * WooCommerce: Remove the tags from the single product view
        */
        if ( get_option( 'tag_group_remove_the_product_tags', false ) ) {

          // call late so that WooHook is already registered
          add_action( 'wp_enqueue_scripts', array( $tag_group_posts, 'remove_the_product_terms' ) );

        }

      }

    }


    /**
    * registers the shortcodes with Gutenberg blocks
    *
    * @param void
    * @return object $this
    */
    public function register_shortcodes_and_blocks()
    {

      // Register shortcodes also for admin so that we can remove them with strip_shortcodes in Ajax call
      TagGroups_Premium_Shortcode::register_frontend();

      if ( is_admin() ) {

        /**
        * Ajax-parts of shortcodes
        */
        TagGroups_Premium_Shortcode::register_backend();

      }

      return $this;

    }


    /**
    * registers the CRON routines
    *
    * @param void
    * @return object $this
    */
    public function register_CRON()
    {

      // CRON independent from admin or frontend
      // TagGroups_Premium_Cron::register_identifiers();

      // schedule purging of expired transients
      TagGroups_Premium_Cron::schedule_regular( 'daily', 'tag_groups_purge_expired_transients' );

      return $this;

    }



    /**
    *   Initializes values and prevents errors that stem from wrong values, e.g. based on earlier bugs.
    *   Runs when plugin is activated.
    *
    * @deprecated
    * @param void
    * @return void
    */
    static function on_activation() {

    }


    /**
    * Adds js and css to frontend
    *
    *
    * @param void
    * @return void
    */
    public function enqueue_scripts() {

      /* the following only for shortcode tag_groups_dpf */
      global $post, $tag_groups_premium_fs_sdk;

      $tag_group_shortcode_enqueue_always = get_option( 'tag_group_shortcode_enqueue_always', 1 );

      if ( $tag_groups_premium_fs_sdk->is_plan_or_trial('premium') ) {

        if ( $tag_group_shortcode_enqueue_always ||
        ( ! is_a( $post, 'WP_Post' ) || has_shortcode( $post->post_content, 'tag_groups_dpf' ) ) ) {

          if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

            wp_register_style( 'tag-groups-premium-css-frontend', TAG_GROUPS_PREMIUM_PLUGIN_URL .  '/assets/css/frontend.css', array(), TAG_GROUPS_VERSION );

          } else {

            wp_register_style( 'tag-groups-premium-css-frontend', TAG_GROUPS_PREMIUM_PLUGIN_URL .  '/assets/css/frontend.min.css', array(), TAG_GROUPS_VERSION );

          }

          wp_enqueue_style( 'tag-groups-premium-css-frontend' );

          // wp_enqueue_script( 'jquery-ui-sortable' ); // Do users need to sort tags?

          wp_enqueue_script( 'jquery-ui-autocomplete' );

          wp_register_style( 'tag-groups-tag-editor-css', TAG_GROUPS_PREMIUM_PLUGIN_URL .  '/assets/css/jquery.tag-editor.css', array(), TAG_GROUPS_VERSION );

          wp_enqueue_style( 'tag-groups-tag-editor-css' );

          if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

            wp_register_script( 'tag-groups-tag-editor-js', TAG_GROUPS_PREMIUM_PLUGIN_URL . '/assets/js/jquery.tag-editor.js', array( 'jquery', 'jquery-ui-core' ), TAG_GROUPS_VERSION );

          } else {

            wp_register_script( 'tag-groups-tag-editor-js', TAG_GROUPS_PREMIUM_PLUGIN_URL . '/assets/js/jquery.tag-editor.min.js', array( 'jquery', 'jquery-ui-core' ), TAG_GROUPS_VERSION );

          }

          wp_enqueue_script( 'tag-groups-tag-editor-js' );

          wp_register_script( 'tag-groups-caret-js', TAG_GROUPS_PREMIUM_PLUGIN_URL . '/assets/js/jquery.caret.min.js', array(), TAG_GROUPS_VERSION );

          wp_enqueue_script( 'tag-groups-caret-js' );

          wp_enqueue_style( 'dashicons' );

          wp_register_script( 'sumoselect-js', TAG_GROUPS_PLUGIN_URL . '/assets/js/jquery.sumoselect.min.js', array( 'jquery' ), TAG_GROUPS_VERSION );

          wp_enqueue_script( 'sumoselect-js' );

          wp_register_style( 'sumoselect-css', TAG_GROUPS_PLUGIN_URL .  '/assets/css/sumoselect.css', array(), TAG_GROUPS_VERSION );

          wp_enqueue_style( 'sumoselect-css' );

        }

      }

      if ( $tag_group_shortcode_enqueue_always ||
      ( ! is_a( $post, 'WP_Post' ) || has_shortcode( $post->post_content, 'tag_groups_shuffle_box' ) ) ) {

        if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

          wp_register_style( 'tag-groups-premium-css-frontend', TAG_GROUPS_PREMIUM_PLUGIN_URL .  '/assets/css/frontend.css', array(), TAG_GROUPS_VERSION );

        } else {

          wp_register_style( 'tag-groups-premium-css-frontend', TAG_GROUPS_PREMIUM_PLUGIN_URL .  '/assets/css/frontend.min.css', array(), TAG_GROUPS_VERSION );

        }

        wp_enqueue_style( 'tag-groups-premium-css-frontend' );

        wp_register_script( 'isotope-js', TAG_GROUPS_PREMIUM_PLUGIN_URL . '/assets/js/isotope.pkgd.min.js', array( 'jquery' ), TAG_GROUPS_VERSION );

        wp_enqueue_script( 'isotope-js' );

      }

      if ( $tag_group_shortcode_enqueue_always ||
      ( ! is_a( $post, 'WP_Post' ) || has_shortcode( $post->post_content, 'tag_groups_table' ) ) ) {

        wp_register_style( 'basictable-css', TAG_GROUPS_PREMIUM_PLUGIN_URL .  '/assets/css/basictable.css', array(), TAG_GROUPS_VERSION );

        wp_enqueue_style( 'basictable-css' );

        wp_register_script( 'basictable-js', TAG_GROUPS_PREMIUM_PLUGIN_URL . '/assets/js/jquery.basictable.min.js', array( 'jquery' ), TAG_GROUPS_VERSION );

        wp_enqueue_script( 'basictable-js' );

      }

      if ( $tag_group_shortcode_enqueue_always ||
      ( ! is_a( $post, 'WP_Post' ) || has_shortcode( $post->post_content, 'tag_groups_menu' ) ) ) {

        wp_enqueue_script( 'jquery' );

      }

      if ( $tag_groups_premium_fs_sdk->is_plan_or_trial('premium') ) {

        if ( $tag_group_shortcode_enqueue_always ||
        ( ! is_a( $post, 'WP_Post' ) || has_shortcode( $post->post_content, 'tag_groups_post_list' ) ) ) {

          if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

            wp_register_style( 'tag-groups-premium-css-frontend', TAG_GROUPS_PREMIUM_PLUGIN_URL .  '/assets/css/frontend.css', array(), TAG_GROUPS_VERSION );

          } else {

            wp_register_style( 'tag-groups-premium-css-frontend', TAG_GROUPS_PREMIUM_PLUGIN_URL .  '/assets/css/frontend.min.css', array(), TAG_GROUPS_VERSION );

          }

          wp_enqueue_style( 'tag-groups-premium-css-frontend' );

        }

      }

      if (
        get_option( 'tag_group_shortcode_enqueue_always', 1 ) || (
          ! is_a( $post, 'WP_Post' ) || (
            has_shortcode( $post->post_content, 'tag_groups_dpf_toggle_menu' ) )
            )
          ) {

            wp_enqueue_script( 'jquery' );

            wp_enqueue_script( 'jquery-ui-core' );

            wp_enqueue_script( 'jquery-ui-accordion' );

          }

        }


        /**
        * Adds js and css to frontend
        *
        *
        * @param string $where
        * @return void
        */
        public function enqueue_scripts_admin( $where ) {

          if ( strpos( $where, 'tag-groups-settings' ) !== false ) {

            wp_enqueue_script( 'jquery' );

            wp_enqueue_script( 'jquery-ui-core' );

            wp_enqueue_script( 'jquery-ui-accordion' );

            if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

              wp_register_style( 'tag-groups-css-backend-tgp', TAG_GROUPS_PREMIUM_PLUGIN_URL .  '/assets/css/backend.css', array(), TAG_GROUPS_VERSION );

            } else {

              wp_register_style( 'tag-groups-css-backend-tgp', TAG_GROUPS_PREMIUM_PLUGIN_URL .  '/assets/css/backend.min.css', array(), TAG_GROUPS_VERSION );

            }

            wp_enqueue_style( 'tag-groups-css-backend-tgp' );

            wp_register_style( 'tag-groups-css-backend-tgp-structure', TAG_GROUPS_PLUGIN_URL . '/assets/css/jquery-ui.structure.min.css', array(), TAG_GROUPS_VERSION );

            wp_enqueue_style( 'tag-groups-css-backend-tgp-structure' );


          } elseif ( strpos( $where, '_page_tag-groups' ) !== false ) {

            wp_enqueue_script( 'jquery' );

            wp_enqueue_script( 'jquery-ui-core' );

            wp_enqueue_script( 'jquery-ui-sortable' );

          } elseif ( strpos( $where, 'post-new.php' ) !== false || strpos( $where, 'post.php' ) !== false ) {

            wp_enqueue_script( 'jquery' );

            wp_enqueue_script( 'jquery-ui-core' );

            wp_enqueue_script( 'jquery-ui-sortable' );

            wp_enqueue_script( 'jquery-ui-autocomplete' );

            if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

              wp_register_style( 'tag-groups-css-backend-tgp', TAG_GROUPS_PREMIUM_PLUGIN_URL .  '/assets/css/backend.css', array(), TAG_GROUPS_VERSION );

            } else {

              wp_register_style( 'tag-groups-css-backend-tgp', TAG_GROUPS_PREMIUM_PLUGIN_URL .  '/assets/css/backend.min.css', array(), TAG_GROUPS_VERSION );

            }

            wp_enqueue_style( 'tag-groups-css-backend-tgp' );

            wp_register_style( 'tag-groups-tag-editor-css', TAG_GROUPS_PREMIUM_PLUGIN_URL .  '/assets/css/jquery.tag-editor.css', array(), TAG_GROUPS_VERSION );

            wp_enqueue_style( 'tag-groups-tag-editor-css' );

            if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

              wp_register_script( 'tag-groups-tag-editor-js', TAG_GROUPS_PREMIUM_PLUGIN_URL . '/assets/js/jquery.tag-editor.js', array( 'jquery', 'jquery-ui-core' ), TAG_GROUPS_VERSION );

            } else {

              wp_register_script( 'tag-groups-tag-editor-js', TAG_GROUPS_PREMIUM_PLUGIN_URL . '/assets/js/jquery.tag-editor.min.js', array( 'jquery', 'jquery-ui-core' ), TAG_GROUPS_VERSION );

            }

            wp_enqueue_script( 'tag-groups-tag-editor-js' );

            wp_register_script( 'tag-groups-caret-js', TAG_GROUPS_PREMIUM_PLUGIN_URL . '/assets/js/jquery.caret.min.js', array(), TAG_GROUPS_VERSION );

            wp_enqueue_script( 'tag-groups-caret-js' );

          }

        }


        /**
        * Checks if we need to run routines for version update
        *
        * @param void
        * @return void
        */
        public function maybe_run_version_update()
        {

          if ( version_compare( TAG_GROUPS_VERSION, get_option( 'tag_group_premium_version', '1.0' ) , '>' ) ) {

            if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

              error_log( '[Tag Groups Premium] version update detected' );

            }

            if ( version_compare( get_option( 'tag_group_premium_version', '1.0' ), '1.18.0' , '<' ) ) {

              if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

                error_log( '[Tag Groups Premium] Schedule migration to new format' );

              }

              TagGroups_Premium_Cron::schedule_in_secs( 5, 'tag_groups_run_migration_new_format' );

            }

            $this->run_version_update();

          }

        }


        /**
        * Runs processes for version number steps
        *
        * @param void
        * @return void
        */
        private function run_version_update()
        {

          global $tag_group_terms;

          /**
          * clear transitions
          */
          $tag_group_terms->clear_term_cache();

          /**
          * clear cache
          */
          if ( class_exists( 'TagGroups_Cache' ) ) {

            $tag_group_object_cache = get_option( 'tag_group_object_cache', TagGroups_Cache::WP_OPTIONS );

            $cache = new TagGroups_Cache();
            $cache->type( $tag_group_object_cache )
            ->path( WP_CONTENT_DIR . '/chatty-mango/cache/' )
            ->purge_all();

          }

          // save new version
          update_option( 'tag_group_premium_version', TAG_GROUPS_VERSION );

        }


      }

    }
