<?php
/**
* Tag Groups Premium
*
* @package     Tag Groups Premium
* @author      Christoph Amthor
* @copyright   2018 Christoph Amthor (@ Chatty Mango, chattymango.com)
* @license     see official vendor website
* @since      1.8.0
*
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*
*/

if ( ! class_exists('TagGroups_Premium_Group') ) {

  class TagGroups_Premium_Group {

    /**
    * term_group id
    *
    * @var int
    */
    private $group_id;

    /**
    * object of groups
    *
    * @var array
    */
    private $tg_premium_groups;


    /**
    * Constructor
    *
    *
    * @param int $group_id optional term_group
    * @return return type
    */
    public function __construct( $group_id = null )
    {

      global $tag_group_premium_groups;

      $this->tg_premium_groups = $tag_group_premium_groups;

      if ( isset( $group_id ) ) {

        $this->group_id = $group_id;

      }

    }


    /**
    * getter for the term_group value
    *
    *
    * @param void
    * @return int term_group
    */
    public function get_group_id()
    {

      return $this->group_id;

    }


    /**
    * setter for the term_group value
    *
    *
    * @param int $group_id
    * @return object $this
    */
    public function set_group_id( $group_id )
    {

      $this->group_id = $group_id;

      return $this;

    }


    /**
    * returns all terms that are associated with this term group, enabled for multiple groups
    *
    * @deprecated since 1.24.0
    * @param int $term_group
    * @param string|array $taxonomy See get_terms
    * @param boolean $hide_empty See get_terms
    * @param string $fields See get_terms
    * @return array|boolean array of WP Term objects, or false
    */
    public function get_group_terms_premium( $taxonomy = 'post_tag', $hide_empty = false, $fields = 'all', $post_id = false, $orderby = 'name', $order = 'ASC' ) {

      /**
      * In case we use the Polylang plugin: get the terms for the language of that post.
      * Polylang needs extra query parameter
      */
      if ( function_exists( 'pll_get_post_language' ) && $post_id ) {

        /**
        * Better sanitize what we get from other plugins
        */
        $pll_post_language = sanitize_text_field( pll_get_post_language( $post_id, 'locale' ) );

      } else {

        $pll_post_language = '';

      }

      /**
      * In case we use the WPML plugin: consider the language
      */
      if ( defined( 'ICL_LANGUAGE_CODE' ) ) {

        $wpml_language = (string) ICL_LANGUAGE_CODE;

      } else {

        $wpml_language = '';

      }

      /**
      * try to get cached version
      *
      * We need to supply the language parameters, because we don't use the Caching class
      */
      $key = md5( $this->group_id . '-' . serialize( $taxonomy ) . '-' . serialize( $hide_empty ) . '-' . strtolower( $fields ) . '-' . $post_id . '-' . strtolower( $orderby ) . '-' . strtolower( $order ) . '-' . $pll_post_language . '-' . $wpml_language );

      $transient_value = get_transient( TagGroups_WPML::get_tag_groups_group_terms_transient_name() );

      $natural = false;

      if ( $transient_value === false || ! isset( $transient_value[ $key ] ) ) {

        if ( 'natural' == $orderby ) {

          $orderby = 'name';

          $natural = true;

        }

        if ( 0 == $this->group_id ) {

          $args = array(
            'taxonomy'    => $taxonomy,
            'hide_empty'  => $hide_empty,
            'fields'      => $fields,
            'orderby'     => $orderby,
            'order'       => $order,
            'meta_query'  => array(
              'relation' => 'OR',
              array(
                'key'     => '_cm_term_group_array',
                'value'   => ',0,', // WP adds %...% ; see https://core.trac.wordpress.org/browser/tags/5.1/src/wp-includes/class-wp-meta-query.php#L549
                'compare' => 'LIKE'
              ),
              array(
                'key'     => '_cm_term_group_array',
                'compare' => 'NOT EXISTS'
              ),
            ),
          );

        } else {

          $args = array(
            'taxonomy'    => $taxonomy,
            'hide_empty'  => $hide_empty,
            'fields'      => $fields,
            'orderby'     => $orderby,
            'order'       => $order,
            'meta_query'  => array(
              array(
                'key'     => '_cm_term_group_array',
                'value'   => ',' . $this->group_id . ',', // WP adds %...% ; see https://core.trac.wordpress.org/browser/tags/5.1/src/wp-includes/class-wp-meta-query.php#L549
                'compare' => 'LIKE'
              ),
            ),
          );

        }

        /**
        * Add Polylang query parameter
        */
        if ( ! empty( $pll_post_language ) ) {

          $args['lang'] = $pll_post_language;

        }

        $terms = get_terms( $args );

        if ( ! empty( $terms ) ) {

          if ( ! is_array( $transient_value ) ) {

            $transient_value = array();

          }

          if ( $natural ) {

            $terms = $this->tg_premium_groups->natural_sorting( $terms, $order );

          }

          $transient_value[ $key ] = $terms;

          set_transient( TagGroups_WPML::get_tag_groups_group_terms_transient_name(), $transient_value, 7 * DAY_IN_SECONDS );

        }

        return $terms;

      } else {

        return $transient_value[ $key ];

      }

    }


    /**
    * Returns pieces for for terms_clauses(), enabled for multiple groups
    *
    * @deprecated since 1.24.0
    * @param int $group_id
    * @return array
    */
    public function terms_clauses() {

      global $wpdb;

      if ( 0 == $this->group_id ) {
        /**
        * We are searching for unassigned terms.
        */

        $meta_query_args = array(
          'relation' => 'OR',
          array(
            'key'     => '_cm_term_group_array',
            'value'   => ',0,',
            'compare' => '='
          ),
          array(
            'key'     => '_cm_term_group_array',
            'compare' => 'NOT EXISTS'
          )
        );

      } else {
        /**
        * We are searching for terms assigned to a group with $term_group.
        */

        $meta_query_args = array(
          // 'relation' => 'OR',
          array(
            'key'     => '_cm_term_group_array',
            'value'   => ',' . $this->group_id . ',', // WP adds %...% ; see https://core.trac.wordpress.org/browser/tags/5.1/src/wp-includes/class-wp-meta-query.php#L549
            'compare' => 'LIKE'
          ),
        );

      }

      /**
      * Convert the arguments to SQL pieces
      */
      $meta_query = new WP_Meta_Query( $meta_query_args );

      return $meta_query->get_sql(
        'term',
        't',
        'term_id'
      );

    }


  }

}
