<?php
/**
* Tag Groups Premium
*
* @package    Tag Groups Premium
* @author     Christoph Amthor
* @copyright  2018 Christoph Amthor (@ Chatty Mango, chattymango.com)
* @license    see official vendor website
* @since      1.14.0
*
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*
*/

if ( ! class_exists( 'TagGroups_Premium_Cache' ) ) {

  /**
  * The main purpose of this class is to keep all cache-related information in one place.
  *
  */
  class TagGroups_Premium_Cache {


    /**
    * Stores data in the ChattyMango_Cache or WordPress Object Cache
    *
    * @param mixed $data
    * @param string $key
    */
    static function set( $key, $data ) {

      if ( class_exists( 'TagGroups_Cache' ) ) {

        $tag_group_object_cache = get_option( 'tag_group_object_cache', TagGroups_Cache::WP_OPTIONS );

        if ( 9 == $tag_group_object_cache ) {

          $key_localized = self::add_language_key( $key );

          // using WP Object Cache
          wp_cache_set( $key_localized, $data, '', TAG_GROUPS_PREMIUM_CACHE_LIFETIME );

          return;

        }

        if ( $tag_group_object_cache ) {

          $key_localized = self::add_language_key( $key );

          $cache = new TagGroups_Cache();

          $cache->type( $tag_group_object_cache )
          ->path( WP_CONTENT_DIR . '/chatty-mango/cache/' )
          ->lifetime( TAG_GROUPS_PREMIUM_CACHE_LIFETIME )
          ->key( $key_localized )
          ->set( $data );

          return;

        }

      }

    }


    /**
    * Retrieves data from the ChattyMango_Cache or WordPress Object Cache
    *
    * @param string $key
    * @return mixed
    */
    static function get( $dummy, $key ) {

      if ( class_exists( 'TagGroups_Cache' ) ) {

        $tag_group_object_cache = get_option( 'tag_group_object_cache', TagGroups_Cache::WP_OPTIONS );

        if ( 9 == $tag_group_object_cache ) {

          $key_localized = self::add_language_key( $key );

          // using WP Object Cache
          return wp_cache_get( $key_localized );

        }

        if ( $tag_group_object_cache ) {

          $key_localized = self::add_language_key( $key );

          $cache = new TagGroups_Cache();

          return $cache->type( $tag_group_object_cache )
          ->path( WP_CONTENT_DIR . '/chatty-mango/cache/' )
          ->key( $key_localized )
          ->get();

        }

      }

      return false;

    }


    /**
    * Deletes entry from the ChattyMango_Cache or WordPress Object Cache
    *
    * @param string $key
    * @return mixed
    */
    static function delete( $key ) {

      if ( class_exists( 'TagGroups_Cache' ) ) {

        $tag_group_object_cache = get_option( 'tag_group_object_cache', TagGroups_Cache::WP_OPTIONS );

        if ( 9 == $tag_group_object_cache ) {

          $key_localized = self::add_language_key( $key );

          // using WP Object Cache
          return wp_cache_delete( $key_localized );

        }

        if ( $tag_group_object_cache ) {

          $key_localized = self::add_language_key( $key );

          $cache = new TagGroups_Cache();

          return $cache->type( $tag_group_object_cache )
          ->path( WP_CONTENT_DIR . '/chatty-mango/cache/' )
          ->key( $key_localized )
          ->delete();

        }

      }

      return false;

    }


    /**
    * Purges all data of the ChattyMango_Cache or WordPress Object Cache
    * @param void
    * @return mixed Result of purging
    */
    static function purge() {

      if ( class_exists( 'TagGroups_Cache' ) ) {

        $tag_group_object_cache = get_option( 'tag_group_object_cache', TagGroups_Cache::WP_OPTIONS );

        if ( 9 == $tag_group_object_cache ) {

          // use WP Object Cache
          return wp_cache_flush();

        }

        if ( $tag_group_object_cache ) {

          try {

            $cache = new TagGroups_Cache();

            return $cache->type( $tag_group_object_cache )
            ->path( WP_CONTENT_DIR . '/chatty-mango/cache/' )
            ->purge_all();

          } catch ( Exception $e ) {

            if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

              error_log( '[Tag Groups Premium] Failed to purge cache.' );

            }

          }

        }

      }

      return false;

    }


    /**
    * Add the current language to the key for caching so that the cache is language-aware.
    *
    *
    * @param $key string
    * @return string
    */
    public static function add_language_key( $key )
    {

      // WPML and Polylang https://polylang.pro/doc/wpml-api/
      if ( defined( 'ICL_LANGUAGE_CODE' ) ) {

        if ( 'all' == ICL_LANGUAGE_CODE ) {

          $language = (string) apply_filters( 'wpml_default_language', NULL );

        } else {

          $language = (string) ICL_LANGUAGE_CODE;

        }

        $key .= "-lang:" . $language;

      }

      return $key;

    }

  }

}
