<?php
/**
* Tag Groups Premium
*
* @package     Tag Groups Premium
* @author      Christoph Amthor
* @copyright   2018 Christoph Amthor (@ Chatty Mango, chattymango.com)
* @license     see official vendor website
* @since      1.8.0
*
*
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
* THE SOFTWARE.
*
*/

if ( ! class_exists( 'TagGroups_Premium_Posts' ) ) {

  class TagGroups_Premium_Posts {


    /**
    * All post terms from transient
    *
    * array(
    *   $post_id_1 = > array(
    *     $group_id_1 => array( $term_id_1, $term_id_2, .. ),
    *     $group_id_2 => array( $term_id_3, $term_id_4, .. ),
    *     ...
    *   ),
    *   $post_id_2 => ...
    * )
    *
    * @var array
    */
    private $all_post_terms;


    /**
    * Constructor
    *
    * @param int $term_group optional term_group
    * @return return type
    */
    public function __construct()
    {

      $this->load();

      return $this;

    }


    /**
    * Load the transient from the database
    *
    * @param void
    * @return object $this
    */
    public function load()
    {

      $this->all_post_terms = get_transient( 'tag_groups_post_terms' );

      return $this;

    }


    /**
    * Setter for $all_post_terms
    *
    * @param array $all_post_terms
    * @return object
    */
    public function set_all_post_terms( $all_post_terms )
    {

      $this->all_post_terms = $all_post_terms;

      return $this;

    }


    /**
    * Getter for $all_post_terms
    *
    * @param void
    * @return array
    */
    public function get_all_post_terms()
    {

      return $this->all_post_terms;

    }


    /**
    * Save the data of all post terms to the transient
    *
    *
    * @param void
    * @return void
    */
    public function save_transient()
    {

      if ( is_array( $this->all_post_terms ) ) {

        set_transient( 'tag_groups_post_terms', $this->remove_empty_items( $this->all_post_terms ), DAY_IN_SECONDS * 7 );

      }

    }


    /**
    * Adds the data for one post to the transient
    *
    *
    * @param int $post_id
    * @param array $terms
    * @return void
    */
    public function add_to_transient( $post_id, $terms ) {

      if ( ! is_array( $this->all_post_terms ) ) {

        $this->all_post_terms = array();

      }

      if ( ! empty( $terms ) ) {

        $this->all_post_terms[ $post_id ] = $terms;

      }

      $this->save_transient();

    }


    /**
    * Convert all term info to the new format for saving with post meta
    *
    * Posts without any tags will be re-processed every time since we cannot exclude them in WP QUERY (without knowing their taxonomies).
    *
    * @param void
    * @return int number of processed items
    */
    public function convert_to_post_meta( $count_only = false, $offset = null, $length = null ) {

      $post_ids = $this->get_post_ids_with_missing_group( null, $count_only );

      if ( $count_only ) {

        if ( is_array( $post_ids ) ) {

          return count( $post_ids );

        } else {

          return 0;

        }

      }

      if ( isset( $offset ) && isset( $length ) ) {

        $post_ids = array_slice( $post_ids, $offset, $length );

      }

      $count = 0;

      $enabled_taxonomies = TagGroups_Taxonomy::get_enabled_taxonomies();

      /**
      * Convert the posts
      */
      if ( is_array( $post_ids ) ) {

        foreach ( $post_ids as $post_id ) {

          $tg_post = new TagGroups_Premium_Post( $post_id );

          $tg_post->create_post_meta_from_tags_and_groups( $enabled_taxonomies );

          $count++;

        }

      }

      return $count;

    }


    /**
    * Converts post meta to new format that is faster to process
    *
    * @param void
    * @return int number of processed items
    */
    public function post_meta_add_comma( $count_only = false, $offset = null, $length = null ) {

      if ( $count_only ) {
        // In case we restarted the tasks, we need to get fresh data

        $post_ids = false;


      } else {

        $post_ids = get_transient( 'tag_groups_post_meta_add_comma_ids' );

      }


      if ( $post_ids === false ) {

        $post_ids = $this->get_post_ids();

        // Try to keep for 10 minutes so that our offset always starts with unprocessed items.
        set_transient( 'tag_groups_post_meta_add_comma_ids', $post_ids, 600 );

      }

      if ( $count_only ) {

        if ( is_array( $post_ids ) ) {

          return count( $post_ids );

        } else {

          return 0;

        }

      }

      if ( isset( $offset ) && isset( $length ) ) {

        $post_ids = array_slice( $post_ids, $offset, $length );

      }

      /**
      * Convert the posts
      */
      $count = 0;

      foreach ( $post_ids as $post_id ) {

        /**
        * get the post meta
        */
        $post_meta_array = get_post_meta( $post_id );

        /**
        * Check for invalid keys in post meta
        */
        foreach ( $post_meta_array as $post_meta_key => $post_meta_values ) {

          if ( strpos( $post_meta_key, '_cm_post_terms_' ) === 0 ) {

            $post_meta_value = TagGroups_Base::get_first_element( $post_meta_values );

            if ( ( substr( $post_meta_value, 0, 1 ) != ',' || substr( $post_meta_value, -1, 1 ) != ',' ) ) {

              $post_meta_array = explode( ',', $post_meta_value );

              // remove empty values
              $post_meta_array = array_filter( $post_meta_array, function( $v ) { return $v != ''; } );

              $result = update_post_meta( $post_id, $post_meta_key, ',' . implode( ',', $post_meta_array ) . ',' );

              if ( $result ) {

                $count++;

              }

            }

          }

        }

      }

      return $count;

    }


    /**
    * Changes the corresponding post meta values after a term has changed its group affiliation
    *
    *
    * @param int $term_id modified term
    * @param array $new_group_ids
    * @return return type
    */
    public function update_post_meta_for_term( $term_id = 0, $new_group_ids )
    {

      global $tag_group_groups;

      if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

        error_log( sprintf( '[Tag Groups Premium] Checking if posts need to be migrated for term ID %d.', $term_id ) );

        $start_time = microtime( true );

      }


      if ( ! is_array( $new_group_ids ) ) {

        $new_group_ids = array( $new_group_ids );

      }


      $all_post_terms = get_transient( 'tag_groups_post_terms' );

      if ( ! is_array( $all_post_terms ) ) {

        $all_post_terms = array();

      }

      /**
      * Check if term was unassigned
      */
      if ( in_array( 0, $new_group_ids ) ) {

        $term_was_unassigned = true;

      } else {

        $term_was_unassigned = false;

      }

      $enabled_taxonomies = TagGroups_Taxonomy::get_enabled_taxonomies();

      $all_term_groups = $tag_group_groups->get_group_ids_by_position();

      $post_ids = $this->get_post_ids_matching_term( $term_id );

      $count = 0;


      /**
      * Loop through all posts that match the post types of enabled taxonomies
      */
      if ( is_array( $post_ids ) ) {

        foreach ( $post_ids as $post_id ) {

          if ( empty( $all_post_terms[ $post_id ] ) ) {
            // not in the transient - need to create

            $post_o = new TagGroups_Premium_Post( $post_id );

            $all_post_terms[ $post_id ] = $post_o->get_terms_by_group();

          }

          $changed_meta = array();

          $post_meta = get_post_meta( $post_id );

          if ( $term_was_unassigned ) {
            /**
            * case a): Term was unassigned from all groups.
            *
            */

            /**
            * 1. add to unassigned group
            */
            // Make sure we don't reduce the $post_meta array by taking the first element
            if ( isset( $post_meta[ '_cm_post_terms_0' ] ) ) {

              $post_meta_array = explode( ',', TagGroups_Base::get_first_element( $post_meta[ '_cm_post_terms_0' ] ) );

              // remove empty values
              $post_meta_array = array_filter( $post_meta_array, function( $v ) { return $v != ''; } );

            } else {

              $post_meta_array = array();

            }

            if ( ! in_array( $term_id, $post_meta_array ) ) {

              $post_meta_array[] = $term_id;

              update_post_meta( $post_id, '_cm_post_terms_0', ',' . implode( ',', $post_meta_array ) . ',' );

            }

            /**
            * 2. remove from all other groups
            */
            foreach ( $all_term_groups as $term_group ) {

              if ( $term_group != 0 && isset( $post_meta[ '_cm_post_terms_' . $term_group ] ) ) {

                // Make sure we don't reduce the $post_meta array by taking the first element
                $post_meta_array = explode( ',', TagGroups_Base::get_first_element( $post_meta[ '_cm_post_terms_' . $term_group ] ) );

                // remove empty values
                $post_meta_array = array_filter( $post_meta_array, function( $v ) { return $v != ''; } );

                if ( in_array( $term_id, $post_meta_array ) ) {

                  $post_meta_array = $this->remove_elements_from_array( $post_meta_array, $term_id );

                  update_post_meta( $post_id, '_cm_post_terms_' . $term_group, ',' . implode( ',', $post_meta_array ) . ',' );

                }

              }

            }

            unset( $all_post_terms[ $post_id ] ); // simply unset, will be recreated

            $count++;

          } else {
            /**
            * case b): Term was assigned to groups that are different than "not assigned".
            *
            */

            /**
            * Don't add tag to post meta if it was already part of that post in a different group.
            *
            * 1. Check if the tag appears in any post meta of that post.
            */
            $term_appears_in_post_meta = false;

            foreach ( $all_term_groups as $term_group ) {

              // We don't include 0 (unassigned) because if a tag appears here we can safely move it to other group
              if ( $term_group != 0 && isset( $post_meta[ '_cm_post_terms_' . $term_group ] ) ) {

                // Make sure we don't reduce the $post_meta array by taking the first element
                $post_meta_array = explode( ',', TagGroups_Base::get_first_element( $post_meta[ '_cm_post_terms_' . $term_group ] ) );

                if ( in_array( $term_id, $post_meta_array ) ) {

                  $term_appears_in_post_meta = true;

                  break;

                }

              }

            }

            /**
            * Loop through all groups, as post meta is segmented by groups
            */
            foreach ( $all_term_groups as $term_group ) {

              if ( isset( $post_meta[ '_cm_post_terms_' . $term_group ] ) ) {
                /**
                * case a): We already have a meta entry for that group.
                */

                // Make sure we don't reduce the $post_meta array by taking the first element
                $post_meta_array = explode( ',', TagGroups_Base::get_first_element( $post_meta[ '_cm_post_terms_' . $term_group ] ) );

                // remove empty values
                $post_meta_array = array_filter( $post_meta_array, function( $v ) { return $v != ''; } );

                /**
                * If this is the "Unassigned" group, then remove from here if tag is in any other group
                * We already made sure before that 0 is not in $new_group_ids.
                */
                if ( 0 == $term_group && ! empty( $new_group_ids ) ) {

                  $changed_meta[ 0 ] = $this->remove_elements_from_array( $post_meta_array, $term_id );

                }
                /**
                * Remove tag from post meta if not supposed to be here
                */
                elseif ( ! in_array( $term_group, $new_group_ids ) && in_array( $term_id, $post_meta_array ) ) {

                  $changed_meta[ $term_group ] = $this->remove_elements_from_array( $post_meta_array, $term_id );

                }

              } else {
                /**
                * case b): We don't have a meta entry for that group.
                */

                /**
                * Add tag if in that group
                *
                * 2. Make sure that the term didn't appear in the post meta.
                */
                if ( in_array( $term_group, $new_group_ids ) && ! $term_appears_in_post_meta ) {

                  // We are creating a new meta entry
                  $changed_meta[ $term_group ] = array( $term_id );

                }

              }

            }

            /**
            * Save post meta that has been removed or added.
            * Also add meta for unassigned terms so we know we have processed them.
            */
            if ( ! empty( $changed_meta ) ) {

              foreach ( $changed_meta as $group_id => $post_meta_array ) {

                if ( count( $post_meta_array ) > 0 ) {

                  update_post_meta( $post_id, '_cm_post_terms_' . $group_id, ',' . implode( ',', $post_meta_array ) . ',' );

                } else {

                  // We don't save meta entries with empty value.
                  delete_post_meta( $post_id, '_cm_post_terms_' . $group_id );

                }

              }

              // delete entry from tag_groups_post_terms, will be recreated
              unset( $all_post_terms[ $post_id ] );

              $count++;

            }

          }

        }

      }

      set_transient( 'tag_groups_post_terms', $this->remove_empty_items( $all_post_terms ), DAY_IN_SECONDS * 7 );

      if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

        error_log( sprintf( '[Tag Groups Premium] Meta of %d post(s) updated in %d milliseconds.', $count, round( ( microtime( true ) - $start_time ) * 1000 ) ) );

      }

      return $count;

    }


    /**
    * Removes elements from an array that are identified by their values.
    *
    * @param array $array From where to remove the elements
    * @param mixed $value Value of elements that need to be removed; if $value is an array, then all elements will be processed.
    * @return array The initial $array without the removed elements
    */
    public function remove_elements_from_array( $array, $value )
    {

      if ( is_array( $value ) ) {

        $keys = array();

        foreach ( $value as $element ) {

          $keys = array_merge( $keys, array_keys( $array, $element ) );

        }

      } else {

        $keys = array_keys( $array, $value );

      }

      if ( ! empty( $keys ) ) {

        foreach ( $keys as $key ) {

          unset( $array[ $key ] );

        }

      }

      return $array;

    }


    /**
    * Returns the meta post query part to filter the list of posts by a term group
    *
    * @param int $term_group
    * @return array
    */
    public function get_meta_query_group( $term_group ) {

      return array(
        array(
          'key' => '_cm_post_terms_' . $term_group,
          'compare' => 'EXISTS'
        )
      );

    }


    /**
    * Fixes all terms in post meta of all posts
    *
    * @param integer $group_id
    * @param boolean $count_only
    * @param integer $offset
    * @param integer $length
    * @return integer
    */
    public function fix_all_incorrect_post_terms( $group_id = null, $count_only = false, $offset = null, $length = null )
    {

      global $tag_group_groups;

      $count = 0;

      if ( $count_only ) {
        // In case we restarted the tasks, we need to get fresh data

        $all_terms = false;

      } else {

        $all_terms = get_transient( 'tag_groups_all_term_ids', false );

      }


      if ( $all_terms === false ) {

        /**
        * get all terms
        */
        $args = array(
          'taxoomy'     => TagGroups_Taxonomy::get_enabled_taxonomies(),
          'hide_empty'  => false,
          'fields'      => 'ids'
        );

        $all_terms = get_terms( $args );

        // Try to keep for 10 minutes so that our offset always starts with unprocessed items.
        set_transient( 'tag_groups_all_term_ids', $all_terms, 600 );

      }

      $post_ids = $this->get_post_ids_matching_groups();

      if ( $count_only ) {

        if ( is_array( $post_ids ) ) {

          return count( $post_ids );

        } else {

          return 0;

        }

      }


      if ( isset( $offset ) && isset( $length ) ) {

        $post_ids = array_slice( $post_ids, $offset, $length );

      }

      /**
      * get all group ids
      */
      $group_ids = $tag_group_groups->get_group_ids_by_position();

      $enabled_taxonomies = TagGroups_Taxonomy::get_enabled_taxonomies();

      if ( is_array( $post_ids ) ) {

        foreach ( $post_ids as $post_id ) {

          $tg_post = new TagGroups_Premium_Post( $post_id );

          $count += $tg_post->fix_incorrect_post_terms( $enabled_taxonomies, $all_terms, $group_ids );

        }

      }

      return $count;

    }


    /**
    * Clear the transient cache tag_groups_post_terms
    *
    *
    * @param integer $group_id
    * @return void
    */
    public function clear_tag_groups_post_terms( $group_id = null )
    {

      delete_transient( 'tag_groups_post_terms' );

    }


    /**
    * Adds a a table after a post with the post tags, sorted by their tag groups
    *
    *
    * @param string $content
    * @return string
    */
    public function add_groups_and_tags_after_content( $content )
    {

      global $post, $tag_group_groups;

      /**
      * We are treating WooCommerce products separately
      */
      if ( is_object( $post ) && 'product' == $post->post_type && get_option( 'tag_group_remove_the_product_tags', false ) ) {

        return $content;

      }


      $include = get_option( 'tag_group_display_groups_under_posts', $tag_group_groups->get_group_ids() );

      if ( empty( $include ) ) {

        return $content;

      }


      if (
        ( is_single() && get_option( 'tag_group_display_groups_under_posts_single', false ) )
        || ( is_feed() && get_option( 'tag_group_display_groups_under_posts_feed', false ) )
        || ( is_home() && get_option( 'tag_group_display_groups_under_posts_home', false ) )
        || ( is_archive() && get_option( 'tag_group_display_groups_under_posts_archive', false ) )
      ) {

        /**
        * Add the table with the groups and tags
        */
        $content .= TagGroups_Premium_API::post_terms( array( 'return' => 'html', 'include' => $include, 'separator' => get_option( 'tag_group_display_groups_under_posts_separator', '&nbsp;|&nbsp;' ) ) );

      }

      return $content;

    }


    /**
    * Sets the array of tags to empty so that the template won't display tags.
    *
    *
    * @param array $term_links
    * @return array
    */
    public function remove_the_post_terms( $term_links )
    {

      return array();

    }


    /**
    * Replaces the default WooCommerce meta template with own by exchanging the callback in the hook
    *
    *
    * @param void
    * @return void
    */
    public function remove_the_product_terms()
    {

      // global $tag_group_posts;

      remove_action( 'woocommerce_single_product_summary', 'woocommerce_template_single_meta', 40 );

      add_action( 'woocommerce_single_product_summary', array( $this, 'woocommerce_template_single_meta' ), 40 );

    }


    /**
    * Callback for WooCommerce to display modified meta template where tags are omitted
    *
    *
    * @param void
    * @return void
    */
    public function woocommerce_template_single_meta()
    {

      $view = new TagGroups_Premium_View( 'third_party/woocommerce/single-product/meta-no-tags' );

      $view->render();

    }


    /**
    * Prepares and calls the template to display tag groups together with WooCommerce Attributes
    *
    * @param object $product
    * @return void
    */
    public function wc_display_product_attributes( $product )
    {

      if ( ! get_option( 'tag_group_add_product_tags_to_attributes', false ) ) {

        return;

      }

      $attributes = $product->get_attributes();

      $attributes = array_filter( $attributes, 'wc_attributes_array_filter_visible' );

      $tg_post = new TagGroups_Premium_Post( $product->get_id() );

      $groups = $tg_post->get_terms_by_group();

      foreach ( $groups as $group_id => $term_ids ) {

        // skip not assigned terms
        if ( 0 == $group_id ) {

          continue;

        }

        /**
        * Create an object that hold all post tags within a group
        */
        $tg_attribute = new TagGroups_Premium_Post_Group( $group_id, $term_ids );

        if ( $tg_attribute->has_terms() ){

          $attributes[ $group_id ] = $tg_attribute;

        }

      }

      $view = new TagGroups_Premium_View( 'third_party/woocommerce/single-product/product-attributes-with-links' );

      $view->set( array(
        'product'            => $product,
        'attributes'         => $attributes,
        'display_dimensions' => apply_filters( 'wc_product_enable_dimensions_display', $product->has_weight() || $product->has_dimensions() ),
        'separator'         => get_option( 'tag_group_display_groups_under_products_separator', '&nbsp;|&nbsp;' ),
      ) );

      $view->render();

      remove_filter( 'woocommerce_product_additional_information', 'wc_display_product_attributes', 10 );

    }


    /**
    * Enforces the display of the "Additional Information" tab if we want to show tag groups but don't have native WC Attributes
    *
    * @param boolean $show_dimensions
    * @return boolean
    */
    public function wc_display_additional_information( $show_dimensions ) {

      global $post;

      if ( ! is_object( $post ) ) {

        return $show_dimensions;

      }

      $tg_post = new TagGroups_Premium_Post( $post->ID );

      if ( ! $tg_post->has_assigned_terms() ) {

        return $show_dimensions;

      }


      if ( get_option( 'tag_group_add_product_tags_to_attributes', false ) ) {

        return true;

      } else {

        return $show_dimensions;

      }

    }


    /**
    * Checks if post meta needs to be processed, after saving without meta box (MB is off, or quick edit, or API)
    *
    *
    * @param integer $post_id
    * @param object $post WP post object
    * @return void
    */
    public function conditional_process_post( $post_id, $post )
    {

      global $tag_group_groups, $tag_group_premium_terms;

      // Check if our nonce is set => We will process the return values from the Meta Box
      if ( isset( $_POST['tag-groups-posts-meta-box-nonce'] ) ) {

        return $post_id;

      }

      // omit autosaves
      if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) { // apparently not equivalent to wp_is_post_autosave( $post_id )

        return $post_id;

      }

      // omit revisions
      if ( 'revision' == $post->post_type ) {

        return $post_id;

      }

      // Check the user's permissions
      $post_type_object = get_post_type_object( $post->post_type );

      if ( is_object( $post_type_object ) && is_object( $post_type_object->cap ) && ! current_user_can( $post_type_object->cap->edit_post, $post_id ) ) {

        return $post_id;

      }

      // check if there was a multisite switch before
      if ( is_multisite() && ms_is_switched() ) {
        /**
        * since version 1.3: In rare cases, the content of $GLOBALS['_wp_switched_stack'] will never be reset => The post is saved, but without the tags. Solution: undo switching
        * Replacing: return $post_id;
        */
        $GLOBALS['_wp_switched_stack'] = array();
        $GLOBALS['switched'] = false;

      }

      $tag_group_meta_box_taxonomies = get_option( 'tag_group_meta_box_taxonomy', array() );

      if ( ! defined('XMLRPC_REQUEST')
      && ! ( isset( $_POST['action'] ) && 'inline-save' == $_POST['action'] )
      && ! ( isset( $_POST['action'] ) && 'editpost' == $_POST['action'] && empty( $tag_group_meta_box_taxonomies ) ) ) {

        return $post_id;

      }

      $tg_post = new TagGroups_Premium_Post( $post_id );

      $tg_post->update_meta();

      do_action( 'purge_post_list_transients', $post_id, $post );

      return true;

    }


    /**
    * remove items with empty values:
    * - posts without tags
    * - tag groups without tags
    *
    * Notes:
    * - We need to keep the keys
    */
    public function remove_empty_items( $target_array ) {

      foreach ( $target_array as $key => $value ) {

        if ( empty( $value ) ) {

          unset( $target_array[ $key ] );

        } elseif ( is_array( $value ) ) {

          $target_array[ $key ] = $this->remove_empty_items( $value );

          if ( empty( $target_array[ $key ] ) ) {

            unset( $target_array[ $key ] );

          }

        }

      }

      return $target_array;

    }


    /**
    * Returns all post IDs that where at least one group is missing in the post meta
    *
    * Omits deleted posts. Uses tranient cache.
    *
    * @param array $taxonomies
    * @param boolean $refresh
    * @return array
    */
    public function get_post_ids_with_missing_group( $taxonomies = null, $refresh = false )
    {

      if ( $refresh ) {

        $post_ids_transient = false;

      } else {

        $post_ids_transient = get_transient( 'tag_groups_post_ids_missing_group' );

      }

      $enabled_taxonomies = TagGroups_Taxonomy::get_enabled_taxonomies();

      if ( empty( $taxonomies ) ) {

        /**
        * We will need the taxonomies
        */
        $taxonomies = $enabled_taxonomies;

      }

      /**
      * Avoid duplicate cache keys
      */
      asort( $taxonomies );

      /**
      * We need an array with numeric indices, because wp-includes/taxonomy.php will search for $taxonomy[0]
      */
      $taxonomies = array_values( $taxonomies );

      /**
      * We cannot use class TagGroups_Group since it might not be available yet
      */
      $term_groups = get_option( 'term_groups', array() );

      if ( empty( $term_groups ) ) {

        // data from old version
        $term_groups = array_keys( array_flip( get_option( 'tag_group_ids', array() ) ) );

      }

      // Remove term group 0 - this is never saved in post meta
      unset( $term_groups[0] );

      /**
      * Avoid duplicate cache keys
      */
      asort( $term_groups );

      $cache_key = md5( serialize( $term_groups ) . serialize( $taxonomies ) );

      if ( $post_ids_transient !== false && isset( $post_ids_transient[ $cache_key ] ) ) {

        $post_ids = $post_ids_transient[ $cache_key ];

      } else {

        /**
        * Get all relevant post types
        */
        $post_types = TagGroups_Taxonomy::post_types_from_taxonomies( $enabled_taxonomies );

        /**
        * search for posts that don't have any _cm_post_terms_{int} meta entry
        */
        $post_args = array(
          'post_type'   => $post_types,
          'meta_query'  => array(
            'relation'  => 'OR'
          ),
          'fields'      => 'ids',
          'posts_per_page' => -1,
          'post_status' => array( 'publish', 'pending', 'draft', 'future', 'private' ) // we are omitting trashed posts
        );


        foreach ( $term_groups as $term_group ) {

          $post_args['meta_query'][] = array(
            'key' => '_cm_post_terms_' . (int) $term_group,
            'compare' => 'NOT EXISTS'
          );

        }

        $post_ids = get_posts( $post_args );

        if ( ! is_array( $post_ids_transient ) ) {

          $post_ids_transient = array();

        }

        $post_ids_transient[ $cache_key ] = $post_ids;

        set_transient( 'tag_groups_post_ids_missing_group', $post_ids_transient, 600 ); // keep for 10 minutes

      }

      return $post_ids;

    }


    /**
    * Returns all post IDs that have tags for certain tag groups and taxonomies
    *
    * Omits deleted posts. Uses tranient cache. If parameters are not supplied, it uses any group and all enabled taxonomies.
    *
    * @param array $term_groups
    * @param array $taxonomies
    * @return array
    */
    public function get_post_ids_matching_groups( $term_groups = array(), $taxonomies = null )
    {

      $post_ids_transient = get_transient( 'tag_groups_post_ids_groups' );

      $enabled_taxonomies = TagGroups_Taxonomy::get_enabled_taxonomies();

      if ( empty( $taxonomies ) ) {

        /**
        * We will need the taxonomies
        */
        $taxonomies = $enabled_taxonomies;

      }

      /**
      * Avoid duplicate cache keys
      */
      asort( $taxonomies );

      /**
      * We need an array with numeric indices, because wp-includes/taxonomy.php will search for $taxonomy[0]
      */
      $taxonomies = array_values( $taxonomies );

      /**
      * Avoid duplicate cache keys
      */
      asort( $term_groups );

      $cache_key = md5( serialize( $term_groups ) . serialize( $taxonomies ) );

      if ( $post_ids_transient !== false && isset( $post_ids_transient[ $cache_key ] ) ) {

        $post_ids = $post_ids_transient[ $cache_key ];

      } else {

        /**
        * Get all relevant post types
        */
        $post_types = TagGroups_Taxonomy::post_types_from_taxonomies( $enabled_taxonomies );

        /**
        * search for posts
        */
        $post_args = array(
          'post_type'   => $post_types,
          'fields'      => 'ids',
          'posts_per_page' => -1,
          'post_status' => array( 'publish', 'pending', 'draft', 'future', 'private' ) // we are omitting trashed posts
        );

        if ( empty( $term_groups ) ) {

          $post_args['meta_query'] = array(
            'relation'  => 'OR'
          );

        } else {

          $post_args['meta_query'] = array(
            'relation'  => 'AND'
          );

        }


        foreach ( $term_groups as $term_group ) {

          $post_args['meta_query'][] = array(
            'key' => '_cm_post_terms_' . (int) $term_group,
            'compare' => 'EXISTS'
          );

        }

        $post_ids = get_posts( $post_args );

        if ( ! is_array( $post_ids_transient ) ) {

          $post_ids_transient = array();

        }

        $post_ids_transient[ $cache_key ] = $post_ids;

        set_transient( 'tag_groups_post_ids_groups', $post_ids_transient, 180 ); // keep for three minutes

      }

      return $post_ids;

    }


    /**
    * Returns all post IDs that have a certain tag
    *
    * Omits deleted posts. Uses tranient cache.
    *
    * @param int $tag_id
    * @return array
    */
    public function get_post_ids_matching_term( $tag_id = null ) {

      $post_ids_transient = get_transient( 'tag_groups_post_ids_matching_term' );

      $cache_key = md5( serialize( $tag_id ) ); // in case we will also admit array in future

      if ( $post_ids_transient !== false && isset( $post_ids_transient[ $cache_key ] ) ) {

        $post_ids = $post_ids_transient[ $cache_key ];

      } else {

        /**
        * We will need the taxonomies
        */
        $enabled_taxonomies = TagGroups_Taxonomy::get_enabled_taxonomies();

        /**
        * We need an array with numeric indices, because wp-includes/taxonomy.php will search for $taxonomy[0]
        */
        $enabled_taxonomies = array_values( $enabled_taxonomies );

        /**
        * Get all relevant post types
        */
        $post_types_for_enabled_taxonomies = TagGroups_Taxonomy::post_types_from_taxonomies( $enabled_taxonomies );


        $post_args = array(
          'post_type'   => $post_types_for_enabled_taxonomies,
          'fields'      => 'ids',
          'posts_per_page' => -1,
          'post_status' => array( 'publish', 'pending', 'draft', 'future', 'private' ), // we are omitting trashed posts
          'tag_id'      => $tag_id
        );

        $post_ids = get_posts( $post_args );

        if ( ! is_array( $post_ids_transient ) ) {

          $post_ids_transient = array();

        }

        $post_ids_transient[ $cache_key ] = $post_ids;

        set_transient( 'tag_groups_post_ids_matching_term', $post_ids_transient, 180 ); // keep for three minutes

      }

      return $post_ids;

    }


    /**
    * Returns all post IDs
    *
    * Omits deleted posts. No extra caching (done by get_posts)
    *
    * @param void
    * @return array
    */
    public function get_post_ids() {

      /**
      * We will need the taxonomies
      */
      $enabled_taxonomies = TagGroups_Taxonomy::get_enabled_taxonomies();

      /**
      * We need an array with numeric indices, because wp-includes/taxonomy.php will search for $taxonomy[0]
      */
      $enabled_taxonomies_values = array_values( $enabled_taxonomies );

      /**
      * Get all relevant post types
      */
      $post_types = TagGroups_Taxonomy::post_types_from_taxonomies( $enabled_taxonomies_values );

      /**
      * search for posts that have a _cm_post_terms_{int} meta entry
      */
      $post_args = array(
        'post_type'   => $post_types,
        'fields'      => 'ids',
        'posts_per_page' => -1,
        'post_status' => array( 'publish', 'pending', 'draft', 'future', 'private' ) // we are omitting trashed posts
      );

      return get_posts( $post_args );

    }


    /**
    * Updates the post meta
    *
    *
    * @param type var Description
    * @return return type
    */
    public static function update_post_meta( $term_id, $term_groups = array() )
    {

      global $tag_group_posts;

      /**
      * update the post meta
      */
      if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

        error_log( '[Tag Groups Premium] Checking if posts need to be migrated.' );

        $start_time = microtime( true );

      }

      $count = $tag_group_posts->update_post_meta_for_term( $term_id, $term_groups );

      if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {

        error_log( sprintf( '[Tag Groups Premium] Meta of %d post(s) updated in %d milliseconds.', $count, round( ( microtime( true ) - $start_time ) * 1000 ) ) );

      }

    }


  }

}
